diff --git a/core/lib/Drupal/Core/Config/Entity/ConfigStorageController.php b/core/lib/Drupal/Core/Config/Entity/ConfigStorageController.php
index 234ad2a..4a57339 100644
--- a/core/lib/Drupal/Core/Config/Entity/ConfigStorageController.php
+++ b/core/lib/Drupal/Core/Config/Entity/ConfigStorageController.php
@@ -80,7 +80,6 @@ class ConfigStorageController extends EntityStorageControllerBase implements Con
   public function __construct(EntityTypeInterface $entity_type, ConfigFactoryInterface $config_factory, StorageInterface $config_storage, UuidInterface $uuid_service) {
     parent::__construct($entity_type);
 
-    $this->idKey = $this->entityType->getKey('id');
     $this->statusKey = $this->entityType->getKey('status');
 
     $this->configFactory = $config_factory;
@@ -228,7 +227,7 @@ public function create(array $values = array()) {
     $class::preCreate($this, $values);
 
     // Set default language to site default if not provided.
-    $values += array('langcode' => language_default()->id);
+    $values += array($this->languageKey => language_default()->id);
 
     $entity = new $class($values, $this->entityTypeId);
     // Mark this entity as new, so isNew() returns TRUE. This does not check
diff --git a/core/lib/Drupal/Core/Entity/ContentEntityBase.php b/core/lib/Drupal/Core/Entity/ContentEntityBase.php
index 82e5241..417f77b 100644
--- a/core/lib/Drupal/Core/Entity/ContentEntityBase.php
+++ b/core/lib/Drupal/Core/Entity/ContentEntityBase.php
@@ -80,6 +80,13 @@
   protected $languages;
 
   /**
+   * The language entity key.
+   *
+   * @var string
+   */
+  protected $languageKey;
+
+  /**
    * Language code identifying the entity active language.
    *
    * This is the language field accessors will use to determine which field
@@ -134,7 +141,9 @@
   public function __construct(array $values, $entity_type, $bundle = FALSE, $translations = array()) {
     $this->entityTypeId = $entity_type;
     $this->bundle = $bundle ? $bundle : $this->entityTypeId;
-    $this->languages = language_list(Language::STATE_ALL);
+    $this->languages = \Drupal::languageManager()->getLanguages(Language::STATE_ALL);
+    $this->languageKey = $this->getEntityType()->getKey('language');
+
 
     foreach ($values as $key => $value) {
       // If the key matches an existing property set the value to the property
@@ -439,7 +448,7 @@ protected function getTranslatedField($name, $langcode) {
    */
   public function set($name, $value, $notify = TRUE) {
     // If default language changes we need to react to that.
-    $notify = $name == 'langcode';
+    $notify = ($name == $this->languageKey);
     $this->get($name)->setValue($value, $notify);
   }
 
@@ -559,7 +568,7 @@ public function language() {
    */
   protected function setDefaultLangcode() {
     // Get the language code if the property exists.
-    if ($this->hasField('langcode') && ($item = $this->get('langcode')) && isset($item->language)) {
+    if ($this->hasField($this->languageKey) && ($item = $this->get($this->languageKey)) && isset($item->language)) {
       $this->defaultLangcode = $item->language->id;
     }
     if (empty($this->defaultLangcode)) {
@@ -568,8 +577,8 @@ protected function setDefaultLangcode() {
     }
     // This needs to be initialized manually as it is skipped when instantiating
     // the language field object to avoid infinite recursion.
-    if (!empty($this->fields['langcode'])) {
-      $this->fields['langcode'][Language::LANGCODE_DEFAULT]->setLangcode($this->defaultLangcode);
+    if (!empty($this->fields[$this->languageKey])) {
+      $this->fields[$this->languageKey][Language::LANGCODE_DEFAULT]->setLangcode($this->defaultLangcode);
     }
   }
 
@@ -591,7 +600,7 @@ protected function updateFieldLangcodes($langcode) {
    * {@inheritdoc}
    */
   public function onChange($name) {
-    if ($name == 'langcode') {
+    if ($name == $this->languageKey) {
       $this->setDefaultLangcode();
       if (isset($this->translations[$this->defaultLangcode])) {
         $message = format_string('A translation already exists for the specified language (@langcode).', array('@langcode' => $this->defaultLangcode));
@@ -713,8 +722,11 @@ public function addTranslation($langcode, array $values = array()) {
 
     // Instantiate a new empty entity so default values will be populated in the
     // specified language.
-    $entity_type = $this->getEntityType();
-    $default_values = array($entity_type->getKey('bundle') => $this->bundle, 'langcode' => $langcode);
+    $info = $this->getEntityType();
+    $default_values = array(
+      $info->getKey('bundle') => $this->bundle,
+      $this->languageKey => $langcode,
+    );
     $entity = \Drupal::entityManager()
       ->getStorageController($this->getEntityTypeId())
       ->create($default_values);
diff --git a/core/lib/Drupal/Core/Entity/DatabaseStorageController.php b/core/lib/Drupal/Core/Entity/DatabaseStorageController.php
index 2056ae1..3bbabaa 100644
--- a/core/lib/Drupal/Core/Entity/DatabaseStorageController.php
+++ b/core/lib/Drupal/Core/Entity/DatabaseStorageController.php
@@ -82,12 +82,6 @@ public function __construct(EntityTypeInterface $entity_type, Connection $databa
 
     $this->database = $database;
     $this->uuidService = $uuid_service;
-
-    // Check if the entity type supports IDs.
-    $this->idKey = $this->entityType->getKey('id');
-
-    // Check if the entity type supports UUIDs.
-    $this->uuidKey = $this->entityType->getKey('uuid');
   }
 
   /**
diff --git a/core/lib/Drupal/Core/Entity/Entity.php b/core/lib/Drupal/Core/Entity/Entity.php
index 27f9895..f0e5909 100644
--- a/core/lib/Drupal/Core/Entity/Entity.php
+++ b/core/lib/Drupal/Core/Entity/Entity.php
@@ -262,7 +262,8 @@ public function access($operation = 'view', AccountInterface $account = NULL) {
    * {@inheritdoc}
    */
   public function language() {
-    $language = language_load($this->langcode);
+    $langcode = $this->{$this->getEntityType()->getKey('language')};
+    $language = \Drupal::languageManager()->getLanguage($langcode);
     if (!$language) {
       // Make sure we return a proper language object.
       $language = new Language(array('id' => Language::LANGCODE_NOT_SPECIFIED));
@@ -271,6 +272,17 @@ public function language() {
   }
 
   /**
+   * Returns the current langcode.
+   *
+   * @todo Add this to interface.
+   *
+   * @return string
+   */
+  public function langcode() {
+    return $this->language()->id;
+  }
+
+  /**
    * {@inheritdoc}
    */
   public function save() {
diff --git a/core/lib/Drupal/Core/Entity/EntityManager.php b/core/lib/Drupal/Core/Entity/EntityManager.php
index c7c111c..90b6b21 100644
--- a/core/lib/Drupal/Core/Entity/EntityManager.php
+++ b/core/lib/Drupal/Core/Entity/EntityManager.php
@@ -346,10 +346,10 @@ protected function buildBaseFieldDefinitions($entity_type_id) {
     $this->moduleHandler->alter('entity_base_field_info', $base_field_definitions, $entity_type);
 
     // Ensure all basic fields are not defined as translatable.
-    $keys = array_intersect_key(array_filter($entity_type->getKeys()), array_flip(array('id', 'revision', 'uuid', 'bundle')));
-    $untranslatable_fields = array_flip(array('langcode') + $keys);
+    $untranslatable_keys = array_flip(array('id', 'revision', 'uuid', 'bundle', 'language'));
+    $untranslatable_fields = array_intersect_key(array_filter($entity_type->getKeys()), $untranslatable_keys);
     foreach ($base_field_definitions as $field_name => $definition) {
-      if (isset($untranslatable_fields[$field_name]) && $definition->isTranslatable()) {
+      if (in_array($field_name, $untranslatable_fields) && $definition->isTranslatable()) {
         throw new \LogicException(String::format('The @field field cannot be translatable.', array('@field' => $definition->getLabel())));
       }
     }
diff --git a/core/lib/Drupal/Core/Entity/EntityStorageControllerBase.php b/core/lib/Drupal/Core/Entity/EntityStorageControllerBase.php
index 7c39f19..93c4a1b 100644
--- a/core/lib/Drupal/Core/Entity/EntityStorageControllerBase.php
+++ b/core/lib/Drupal/Core/Entity/EntityStorageControllerBase.php
@@ -6,6 +6,7 @@
  */
 
 namespace Drupal\Core\Entity;
+
 use Drupal\Core\Entity\Query\QueryInterface;
 use Drupal\Core\Extension\ModuleHandlerInterface;
 use Symfony\Component\DependencyInjection\ContainerInterface;
@@ -60,6 +61,13 @@
   protected $uuidKey;
 
   /**
+   * The name of the entity langcode property.
+   *
+   * @var string
+   */
+  protected $languageKey;
+
+  /**
    * Constructs an EntityStorageControllerBase instance.
    *
    * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
@@ -70,6 +78,10 @@ public function __construct(EntityTypeInterface $entity_type) {
     $this->entityType = $entity_type;
     // Check if the entity type supports static caching of loaded entities.
     $this->cache = $this->entityType->isStaticallyCacheable();
+
+    $this->idKey = $this->entityType->getKey('id');
+    $this->uuidKey = $this->entityType->getKey('uuid');
+    $this->languageKey = $this->entityType->getKey('language');
   }
 
   /**
diff --git a/core/lib/Drupal/Core/Entity/EntityType.php b/core/lib/Drupal/Core/Entity/EntityType.php
index 4eae0df..4676642 100644
--- a/core/lib/Drupal/Core/Entity/EntityType.php
+++ b/core/lib/Drupal/Core/Entity/EntityType.php
@@ -229,7 +229,11 @@ public function isFieldDataCacheable() {
    * {@inheritdoc}
    */
   public function getKeys() {
-    return $this->entity_keys + array('revision' => '', 'bundle' => '');
+    return $this->entity_keys + array(
+      'revision' => '',
+      'bundle' => '',
+      'language' => '',
+    );
   }
 
   /**
diff --git a/core/lib/Drupal/Core/Entity/FieldableDatabaseStorageController.php b/core/lib/Drupal/Core/Entity/FieldableDatabaseStorageController.php
index eaa4951..db7c00d 100644
--- a/core/lib/Drupal/Core/Entity/FieldableDatabaseStorageController.php
+++ b/core/lib/Drupal/Core/Entity/FieldableDatabaseStorageController.php
@@ -279,7 +279,8 @@ protected function attachPropertyData(array &$entities) {
 
         // Field values in default language are stored with
         // Language::LANGCODE_DEFAULT as key.
-        $langcode = empty($values['default_langcode']) ? $values['langcode'] : Language::LANGCODE_DEFAULT;
+        // @todo Consider adding a 'default_language' key.
+        $langcode = empty($values["default_{$this->languageKey}"]) ? $values[$this->languageKey] : Language::LANGCODE_DEFAULT;
         $translations[$id][$langcode] = TRUE;
 
         foreach (array_keys($field_definitions) as $field_name) {
@@ -357,13 +358,15 @@ protected function buildPropertyQuery(QueryInterface $entity_query, array $value
       //   apply to the default language. See http://drupal.org/node/1866330.
       // Default to the original entity language if not explicitly specified
       // otherwise.
-      if (!array_key_exists('default_langcode', $values)) {
-        $values['default_langcode'] = 1;
+      // @todo Consider adding a 'default_language' key.
+      $default_language_key = "default_{$this->languageKey}";
+      if (!array_key_exists($default_language_key, $values)) {
+        $values[$default_language_key] = 1;
       }
       // If the 'default_langcode' flag is explicitly not set, we do not care
       // whether the queried values are in the original entity language or not.
-      elseif ($values['default_langcode'] === NULL) {
-        unset($values['default_langcode']);
+      elseif ($values[$default_language_key] === NULL) {
+        unset($values[$default_language_key]);
       }
     }
 
diff --git a/core/modules/aggregator/lib/Drupal/aggregator/Entity/Feed.php b/core/modules/aggregator/lib/Drupal/aggregator/Entity/Feed.php
index cf467da..2212c23 100644
--- a/core/modules/aggregator/lib/Drupal/aggregator/Entity/Feed.php
+++ b/core/modules/aggregator/lib/Drupal/aggregator/Entity/Feed.php
@@ -10,6 +10,7 @@
 use Drupal\Core\Entity\ContentEntityBase;
 use Drupal\Core\Entity\EntityTypeInterface;
 use Drupal\Core\Field\FieldDefinition;
+use Drupal\Core\Language\Language;
 use Symfony\Component\DependencyInjection\Container;
 use Drupal\Core\Entity\EntityStorageControllerInterface;
 use Drupal\aggregator\FeedInterface;
@@ -53,6 +54,32 @@ public function id() {
   }
 
   /**
+   * Overrides ContentEntityBase::langcode().
+   *
+   * Aggregator feeds are not translatable, thus we cannot enable the content
+   * entity translation system by specifying a 'language' key. The language is
+   * specified directly as an entity field.
+   *
+   * @see \Drupal\Core\Entity\ContentEntityBase::langcode()
+   */
+  public function langcode() {
+    return $this->get('langcode')->value;
+  }
+
+  /**
+   * Overrides ContentEntityBase::language().
+   *
+   * Aggregator feeds are not translatable, thus we cannot enable the content
+   * entity translation system by specifying a 'language' key. The language is
+   * specified directly as an entity field.
+   *
+   * @see \Drupal\Core\Entity\ContentEntityBase::language()
+   */
+  public function language() {
+    return \Drupal::languageManager()->getLanguage($this->langcode());
+  }
+
+  /**
    * Implements Drupal\Core\Entity\EntityInterface::label().
    */
   public function label() {
diff --git a/core/modules/aggregator/lib/Drupal/aggregator/Entity/Item.php b/core/modules/aggregator/lib/Drupal/aggregator/Entity/Item.php
index 524e68f..92eb86d 100644
--- a/core/modules/aggregator/lib/Drupal/aggregator/Entity/Item.php
+++ b/core/modules/aggregator/lib/Drupal/aggregator/Entity/Item.php
@@ -41,6 +41,32 @@ public function id() {
   }
 
   /**
+   * Overrides ContentEntityBase::langcode().
+   *
+   * Aggregator items are not translatable, thus we cannot enable the content
+   * entity translation system by specifying a 'language' key. The language is
+   * inherited from the feed and stored directly as an entity field.
+   *
+   * @see \Drupal\Core\Entity\ContentEntityBase::langcode()
+   */
+  public function langcode() {
+    return $this->get('langcode')->value;
+  }
+
+  /**
+   * Overrides ContentEntityBase::language().
+   *
+   * Aggregator items are not translatable, thus we cannot enable the content
+   * entity translation system by specifying a 'language' key. The language is
+   * inherited from the feed and stored directly as an entity field.
+   *
+   * @see \Drupal\Core\Entity\ContentEntityBase::language()
+   */
+  public function language() {
+    return $this->get('langcode')->language;
+  }
+
+  /**
    * Implements Drupal\Core\Entity\EntityInterface::label().
    */
   public function label() {
diff --git a/core/modules/block/custom_block/lib/Drupal/custom_block/Entity/CustomBlock.php b/core/modules/block/custom_block/lib/Drupal/custom_block/Entity/CustomBlock.php
index 88d232b..a2b31e8 100644
--- a/core/modules/block/custom_block/lib/Drupal/custom_block/Entity/CustomBlock.php
+++ b/core/modules/block/custom_block/lib/Drupal/custom_block/Entity/CustomBlock.php
@@ -48,7 +48,8 @@
  *     "revision" = "revision_id",
  *     "bundle" = "type",
  *     "label" = "info",
- *     "uuid" = "uuid"
+ *     "uuid" = "uuid",
+ *     "language" = "langcode",
  *   },
  *   bundle_entity_type = "custom_block_type"
  * )
diff --git a/core/modules/comment/lib/Drupal/comment/Entity/Comment.php b/core/modules/comment/lib/Drupal/comment/Entity/Comment.php
index 6fd4552..00525ba 100644
--- a/core/modules/comment/lib/Drupal/comment/Entity/Comment.php
+++ b/core/modules/comment/lib/Drupal/comment/Entity/Comment.php
@@ -42,7 +42,8 @@
  *     "id" = "cid",
  *     "bundle" = "field_id",
  *     "label" = "subject",
- *     "uuid" = "uuid"
+ *     "uuid" = "uuid",
+ *     "language" = "langcode",
  *   },
  *   links = {
  *     "canonical" = "comment.permalink",
diff --git a/core/modules/content_translation/content_translation.module b/core/modules/content_translation/content_translation.module
index b6f8f5e..4902eaf 100644
--- a/core/modules/content_translation/content_translation.module
+++ b/core/modules/content_translation/content_translation.module
@@ -275,8 +275,19 @@ function _content_translation_menu_strip_loaders($path) {
  *   The entity whose translation overview should be displayed.
  */
 function content_translation_translate_access(EntityInterface $entity) {
-  return $entity instanceof ContentEntityInterface && empty($entity->getUntranslated()->language()->locked) && \Drupal::languageManager()->isMultilingual() && $entity->isTranslatable() &&
-    (user_access('create content translations') || user_access('update content translations') || user_access('delete content translations'));
+  $account = \Drupal::currentUser();
+
+  $access = $entity instanceof ContentEntityInterface;
+  $access = $access && empty($entity->getUntranslated()->language()->locked);
+  $access = $access && \Drupal::languageManager()->isMultilingual();
+  $access = $access && $entity->isTranslatable();
+  $access = $access && (
+    $account->hasPermission('create content translations') ||
+    $account->hasPermission('update content translations') ||
+    $account->hasPermission('delete content translations')
+  );
+
+  return $access;
 }
 
 /**
diff --git a/core/modules/menu_link/lib/Drupal/menu_link/Entity/MenuLink.php b/core/modules/menu_link/lib/Drupal/menu_link/Entity/MenuLink.php
index 36705d5..a3716d1 100644
--- a/core/modules/menu_link/lib/Drupal/menu_link/Entity/MenuLink.php
+++ b/core/modules/menu_link/lib/Drupal/menu_link/Entity/MenuLink.php
@@ -36,7 +36,8 @@
  *     "id" = "mlid",
  *     "label" = "link_title",
  *     "uuid" = "uuid",
- *     "bundle" = "bundle"
+ *     "bundle" = "bundle",
+ *     "language" = "langcode",
  *   },
  * )
  */
diff --git a/core/modules/node/lib/Drupal/node/Entity/Node.php b/core/modules/node/lib/Drupal/node/Entity/Node.php
index a80577c..58a4ef5 100644
--- a/core/modules/node/lib/Drupal/node/Entity/Node.php
+++ b/core/modules/node/lib/Drupal/node/Entity/Node.php
@@ -45,6 +45,7 @@
  *     "id" = "nid",
  *     "revision" = "vid",
  *     "bundle" = "type",
+ *     "language" = "langcode",
  *     "label" = "title",
  *     "uuid" = "uuid"
  *   },
diff --git a/core/modules/shortcut/lib/Drupal/shortcut/Entity/Shortcut.php b/core/modules/shortcut/lib/Drupal/shortcut/Entity/Shortcut.php
index 56d4839..eb1d6bd 100644
--- a/core/modules/shortcut/lib/Drupal/shortcut/Entity/Shortcut.php
+++ b/core/modules/shortcut/lib/Drupal/shortcut/Entity/Shortcut.php
@@ -36,6 +36,7 @@
  *     "id" = "id",
  *     "uuid" = "uuid",
  *     "bundle" = "shortcut_set",
+ *     "language" = "langcode",
  *     "label" = "title"
  *   },
  *   links = {
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTest.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTest.php
index d55bb8d..c58c743 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTest.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTest.php
@@ -37,7 +37,8 @@
  *     "id" = "id",
  *     "uuid" = "uuid",
  *     "bundle" = "type",
- *     "label" = "name"
+ *     "label" = "name",
+ *     "language" = "langcode",
  *   },
  *   links = {
  *     "canonical" = "entity_test.render",
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMul.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMul.php
index 49f7d5f..fa7c385 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMul.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMul.php
@@ -33,6 +33,7 @@
  *     "id" = "id",
  *     "uuid" = "uuid",
  *     "bundle" = "type",
+ *     "language" = "langcode",
  *     "label" = "name"
  *   },
  *   links = {
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMulRev.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMulRev.php
index 870af93..f82bb0e 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMulRev.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestMulRev.php
@@ -34,6 +34,7 @@
  *     "id" = "id",
  *     "uuid" = "uuid",
  *     "revision" = "revision_id",
+ *     "language" = "langcode",
  *     "bundle" = "type"
  *   },
  *   links = {
diff --git a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestRev.php b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestRev.php
index 61753d9..704f50f 100644
--- a/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestRev.php
+++ b/core/modules/system/tests/modules/entity_test/lib/Drupal/entity_test/Entity/EntityTestRev.php
@@ -33,6 +33,7 @@
  *     "revision" = "revision_id",
  *     "bundle" = "type",
  *     "label" = "name",
+ *     "language" = "langcode",
  *   },
  *   links = {
  *     "canonical" = "entity_test.edit_entity_test_rev",
diff --git a/core/modules/taxonomy/lib/Drupal/taxonomy/Entity/Term.php b/core/modules/taxonomy/lib/Drupal/taxonomy/Entity/Term.php
index 38c84e1..f25a21c 100644
--- a/core/modules/taxonomy/lib/Drupal/taxonomy/Entity/Term.php
+++ b/core/modules/taxonomy/lib/Drupal/taxonomy/Entity/Term.php
@@ -40,7 +40,8 @@
  *     "id" = "tid",
  *     "bundle" = "vid",
  *     "label" = "name",
- *     "uuid" = "uuid"
+ *     "uuid" = "uuid",
+ *     "language" = "langcode",
  *   },
  *   bundle_entity_type = "taxonomy_vocabulary",
  *   links = {
diff --git a/core/modules/user/lib/Drupal/user/Entity/User.php b/core/modules/user/lib/Drupal/user/Entity/User.php
index 49c66fd..6697ee5 100644
--- a/core/modules/user/lib/Drupal/user/Entity/User.php
+++ b/core/modules/user/lib/Drupal/user/Entity/User.php
@@ -41,7 +41,8 @@
  *   translatable = TRUE,
  *   entity_keys = {
  *     "id" = "uid",
- *     "uuid" = "uuid"
+ *     "uuid" = "uuid",
+ *     "language" = "langcode",
  *   },
  *   links = {
  *     "canonical" = "user.view",
diff --git a/core/tests/Drupal/Tests/Core/Entity/EntityManagerTest.php b/core/tests/Drupal/Tests/Core/Entity/EntityManagerTest.php
index 2f8c233..24a0ed9 100644
--- a/core/tests/Drupal/Tests/Core/Entity/EntityManagerTest.php
+++ b/core/tests/Drupal/Tests/Core/Entity/EntityManagerTest.php
@@ -544,7 +544,7 @@ public function testGetFieldDefinitionsWithCaching() {
    * @expectedException \LogicException
    */
   public function testGetBaseFieldDefinitionsInvalidDefinition() {
-    $langcode_definition = $this->setUpEntityWithFieldDefinition(FALSE, 'langcode');
+    $langcode_definition = $this->setUpEntityWithFieldDefinition(FALSE, 'langcode', 1, array('language' => 'langcode'));
     $langcode_definition->expects($this->once())
       ->method('isTranslatable')
       ->will($this->returnValue(TRUE));
@@ -560,11 +560,17 @@ public function testGetBaseFieldDefinitionsInvalidDefinition() {
    *   ModuleHandlerInterface::invokeAll() implementation. Defaults to FALSE.
    * @param string $field_definition_id
    *   (optional) The ID to use for the field definition. Defaults to 'id'.
+   * @param int $base_field_definition_calls
+   *   (optional) The number of times EntityInterface::baseFieldDefinitions() is
+   *   expected to be called. Defaults to 1.
+   * @param array $entity_keys
+   *   (optional) An array of entity keys for the mocked entity type. Defaults
+   *   to an empty array.
    *
    * @return \Drupal\Core\Field\FieldDefinition|\PHPUnit_Framework_MockObject_MockObject
    *   A field definition object.
    */
-  protected function setUpEntityWithFieldDefinition($custom_invoke_all = FALSE, $field_definition_id = 'id', $base_field_definition_calls = 1) {
+  protected function setUpEntityWithFieldDefinition($custom_invoke_all = FALSE, $field_definition_id = 'id', $base_field_definition_calls = 1, $entity_keys = array()) {
     $entity_type = $this->getMock('Drupal\Core\Entity\EntityTypeInterface');
     $entity = $this->getMock('Drupal\Tests\Core\Entity\TestContentEntityInterface');
     $entity_class = get_class($entity);
@@ -574,7 +580,7 @@ protected function setUpEntityWithFieldDefinition($custom_invoke_all = FALSE, $f
       ->will($this->returnValue($entity_class));
     $entity_type->expects($this->any())
       ->method('getKeys')
-      ->will($this->returnValue(array()));
+      ->will($this->returnValue($entity_keys));
     $field_definition = $this->getMockBuilder('Drupal\Core\Field\FieldDefinition')
       ->disableOriginalConstructor()
       ->getMock();
diff --git a/core/tests/Drupal/Tests/Core/Entity/EntityTypeTest.php b/core/tests/Drupal/Tests/Core/Entity/EntityTypeTest.php
index 0feb65d..081460d 100644
--- a/core/tests/Drupal/Tests/Core/Entity/EntityTypeTest.php
+++ b/core/tests/Drupal/Tests/Core/Entity/EntityTypeTest.php
@@ -79,9 +79,9 @@ public function testHasKey($entity_keys, $expected) {
    */
   public function providerTestGetKeys() {
     return array(
-      array(array(), array('revision' => '', 'bundle' => '')),
-      array(array('id' => 'id'), array('id' => 'id', 'revision' => '', 'bundle' => '')),
-      array(array('bundle' => 'bundle'), array('bundle' => 'bundle', 'revision' => '')),
+      array(array(), array('revision' => '', 'bundle' => '', 'language' => '')),
+      array(array('id' => 'id'), array('id' => 'id', 'revision' => '', 'bundle' => '', 'language' => '')),
+      array(array('bundle' => 'bundle'), array('bundle' => 'bundle', 'revision' => '', 'language' => '')),
     );
   }
 

diff --git a/core/lib/Drupal/Core/Config/Entity/ConfigEntityType.php b/core/lib/Drupal/Core/Config/Entity/ConfigEntityType.php
index 9a61693..c99ffdd 100644
--- a/core/lib/Drupal/Core/Config/Entity/ConfigEntityType.php
+++ b/core/lib/Drupal/Core/Config/Entity/ConfigEntityType.php
@@ -52,8 +52,11 @@ public function getConfigPrefix() {
    * {@inheritdoc}
    */
   public function getKeys() {
-    // Always add a default 'uuid' key.
-    return array('uuid' => 'uuid') + parent::getKeys();
+    // Always add default 'uuid' and 'language' keys.
+    return array(
+      'uuid' => 'uuid',
+      'language' => 'langcode',
+    ) + parent::getKeys();
   }
 
 